﻿/*###############################################################################
# Copyright (C) MOXA Inc. All rights reserved.
#
# This software is distributed under the terms of the
# MOXA License.  See the file COPYING-MOXA for details.
#
# Purpose : Main Form of the MOXA PnP VPN Cloud
# Author : Jemin Su
# Date : 2016-08-05
# Version : 1.0
# Notes :
##############################################################################*/

using System;
using System.Drawing;
using System.Net.NetworkInformation;
using System.Windows.Forms;

namespace VPNClient
{
    public partial class MainForm : Form
    {
        private VPNCloudService m_service = VPNCloudService.GetInstance();

        private static MainForm m_instance = new MainForm();

        private const int TAB_SIGNIN = 0;
        private const int TAB_DEVICEPOOL = 1;
        private const int TAB_OPTION = 2;
        private const int TAB_USAGE = 3;
        private const int TAB_LOG = 4;
        private const int TAB_LANGUAGE = 5;
        private const int TAB_HELP = 6;
        private const int TAB_END = 7;

        private Button[] m_tabBtn = new Button[MainForm.TAB_END];

        /****************************************
        * Purpose: Interface Combobox item
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-29
        ****************************************/
        private class InterfaceItem
        {
            private NetworkInterface nic;

            public InterfaceItem(NetworkInterface nic)
            {
                this.nic = nic;
            }

            public NetworkInterface getNic()
            {
                return this.nic;
            }

            public override string ToString()
            {
                // Generates the text shown in the combo box
                return this.nic.Name;
            }
        }

        /****************************************
        * Purpose: Constructor for MainForm
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        private MainForm()
        {
            InitializeComponent();

            /* Initial the tab button array */
            this.m_tabBtn[TAB_SIGNIN] = btnSignIn;
            this.m_tabBtn[TAB_DEVICEPOOL] = btnDevicePool;
            this.m_tabBtn[TAB_OPTION] = btnOption;
            this.m_tabBtn[TAB_USAGE] = btnUsage;
            this.m_tabBtn[TAB_LOG] = btnLog;
            this.m_tabBtn[TAB_LANGUAGE] = btnLanguage;
            this.m_tabBtn[TAB_HELP] = btnHelp;

            /* Default selected tab */
            this.tabCtrl.SelectedIndex = TAB_SIGNIN;
            this.m_tabBtn[TAB_SIGNIN].BackColor = Color.CadetBlue;
            this.m_tabBtn[TAB_SIGNIN].ForeColor = Color.White;

            /* Initialize the config loader */
            ConfigLoader loader = ConfigLoader.GetInstance();
            ClientConfig config = loader.GetConfig();
            if(config.start_as_service)
            {
                this.SetServiceMode(true);
            }
            if(config.client_to_client)
            {
                this.SetClientVisible(true);
            }
        }

        /****************************************
        * Purpose: Singleton for MainForm
        * Input: N/A
        * Output: N/A
        * Return: MainForm instance
        * Author: Jemin Su
        * Date: 2017-02-15
        ****************************************/
        public static MainForm GetInstance()
        {
            return m_instance;
        }

        /****************************************
        * Purpose: Set the signed in status
        * Input: status - signed in status
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-15
        ****************************************/
        public void SetSignedIn(bool status)
        {
            if(status == true)
            {
                this.lbStatusSignedIn.BackColor = Color.LightGreen;
            }
            else
            {
                this.lbStatusSignedIn.BackColor = Color.White;
            }
        }

        /****************************************
        * Purpose: Set service mode
        * Input: mode - service mode
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-15
        ****************************************/
        public void SetServiceMode(bool mode)
        {
            if (mode == true)
            {
                this.lbStatusServiceMode.BackColor = Color.LightGreen;
            }
            else
            {
                this.lbStatusServiceMode.BackColor = Color.White;
            }
        }

        /****************************************
        * Purpose: Set the client visibility in status
        * Input: visibility- client visibility
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-16
        ****************************************/
        public void SetClientVisible(bool visibility)
        {
            if (visibility == true)
            {
                this.lbStatusVisible.BackColor = Color.LightGreen;
            }
            else
            {
                this.lbStatusVisible.BackColor = Color.White;
            }
        }

       /****************************************
       * Purpose: Set the client virtual ip
       * Input: ip- client virtual ip
       * Output: N/A
       * Return: N/A
       * Author: Jemin Su
       * Date: 2017-02-16
       ****************************************/
        public void SetVirtualIP(string ip)
        {
            this.lbStatusVirtualIP.Text = ip;         
        }

        /****************************************
        * Purpose: Set the tab index
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        public void SetTabIndex(int index)
        {
            if(index != this.tabCtrl.SelectedIndex)
            {
                int prev = this.tabCtrl.SelectedIndex;
                this.tabCtrl.SelectedIndex = index;

                /* Recover the previous tab button */
                this.m_tabBtn[prev].BackColor = Color.LightGray;
                this.m_tabBtn[prev].ForeColor = Color.White;

                /* Set the selected tab button */
                this.m_tabBtn[index].BackColor = Color.CadetBlue;
                this.m_tabBtn[index].ForeColor = Color.White;
            }
        }

        /****************************************
        * Purpose: Show About Box
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        private void aboutMOXAPnPVPNCloudClientAToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox box = new AboutBox();
            box.ShowDialog();
        }

        /****************************************
        * Purpose: Reload the current interface in this PC
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-26
        ****************************************/
        public void ReloadInterface()
        {
            NetworkInterface[] nics = NetworkInterface.GetAllNetworkInterfaces();

            /* Comment by Jemin for temp
            this.cmbInterface.Items.Clear();
            foreach (var nic in nics)
            {
                if (nic.NetworkInterfaceType == NetworkInterfaceType.Ethernet)
                {
                    this.cmbInterface.Items.Add(new InterfaceItem(nic));
                }
            }*/

            /*String strHostName = Dns.GetHostName();   // 取得本機的 IpHostEntry 類別實體
            IPHostEntry iphostentry = Dns.GetHostEntry(strHostName);// 取得所有 IP 位址
            int num = 1;
            foreach (IPAddress ipaddress in iphostentry.AddressList)
            {
                Console.WriteLine("IP #" + num + ": " + ipaddress.ToString());
                num = num + 1;
            }*/
        }

        /****************************************
        * Purpose: Click the client tree view items
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-22
        ****************************************/
        private void treeClient_Click(object sender, EventArgs e)
        {
            /*int index = this.treeClient.SelectedNode.Index;
            int level = this.treeClient.SelectedNode.Level;
            ListViewItem item = null;
            string vip = null;

            this.listProperty.BeginUpdate();
            this.listProperty.Items.Clear();
            if (index >= 0)
            {
                switch(level)
                {
                    case 0: // Gateway or client                        
                        if(this.m_deviceGroup.Clients[index].type == ClientType.GATEWAY) // Gateway
                        {
                            vip = this.m_service.GetGatewayVirtualIP(
                                this.m_deviceGroup.network,
                                this.m_deviceGroup.Clients[index].network);
                        }
                        else if (this.m_deviceGroup.Clients[index].type == ClientType.CLIENT) // Client
                        {
                            vip = this.m_service.GetClientVirtualIP(
                                this.m_deviceGroup.network,
                                this.m_deviceGroup.Clients[index].network);
                        }

                        item = new ListViewItem("Type");
                        this.listProperty.Items.Add(item);
                        if (this.m_deviceGroup.Clients[index].type == ClientType.GATEWAY)
                            item.SubItems.Add(ClientType.GATEWAY_STRING);
                        else if (this.m_deviceGroup.Clients[index].type == ClientType.CLIENT)
                            item.SubItems.Add(ClientType.CLIENT_STRING);

                        item = new ListViewItem("Name");
                        this.listProperty.Items.Add(item);
                        item.SubItems.Add(this.m_deviceGroup.Clients[index].name);

                        item = new ListViewItem("IP Mapping");
                        this.listProperty.Items.Add(item);
                        if (this.m_deviceGroup.Clients[index].auto_mapping)
                            item.SubItems.Add("On");
                        else
                            item.SubItems.Add("Off");

                        item = new ListViewItem("Virtual IP");
                        this.listProperty.Items.Add(item);
                        item.SubItems.Add(vip);

                        item = new ListViewItem("Status");
                        this.listProperty.Items.Add(item);
                        if (this.m_deviceGroup.Clients[index].status == OnlineStatus.OFFLINE)
                            item.SubItems.Add(OnlineStatus.OFFLINE_STRING);
                        else if(this.m_deviceGroup.Clients[index].status == OnlineStatus.ONLINE)
                            item.SubItems.Add(OnlineStatus.ONLINE_STRING);

                        break;
                    case 1: // Devices
                        int clientIndex = this.treeClient.SelectedNode.Parent.Index;
                        vip = this.m_service.GetDeviceVirtualIP(
                            this.m_deviceGroup.network,
                            this.m_deviceGroup.Clients[clientIndex].network,
                            this.m_deviceGroup.Clients[clientIndex].Devices[index].network);

                        item = new ListViewItem("Type");
                        this.listProperty.Items.Add(item);
                        item.SubItems.Add(ClientType.DEVICE_STRING);

                        item = new ListViewItem("Name");
                        this.listProperty.Items.Add(item);
                        item.SubItems.Add(this.m_deviceGroup.Clients[clientIndex].Devices[index].name);

                        item = new ListViewItem("Virtual IP");
                        this.listProperty.Items.Add(item);
                        item.SubItems.Add(vip);

                        item = new ListViewItem("Local IP");
                        this.listProperty.Items.Add(item);
                        item.SubItems.Add(this.m_deviceGroup.Clients[clientIndex].Devices[index].device_ip);

                        item = new ListViewItem("Status");
                        this.listProperty.Items.Add(item);
                        if (this.m_deviceGroup.Clients[clientIndex].Devices[index].status == OnlineStatus.OFFLINE)
                            item.SubItems.Add(OnlineStatus.OFFLINE_STRING);
                        else if(this.m_deviceGroup.Clients[clientIndex].Devices[index].status == OnlineStatus.ONLINE)
                            item.SubItems.Add(OnlineStatus.ONLINE_STRING);

                        break;
                }
            }
            this.listProperty.EndUpdate();*/
        }

        /****************************************
        * Purpose: Chanage the interface
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-19
        ****************************************/
        private void ckbInterface_SelectedIndexChanged(object sender, EventArgs e)
        {
            /*InterfaceItem selected = this.cmbInterface.SelectedItem as InterfaceItem;
            ListViewItem item = null;

            if(selected != null)
            {
                NetworkInterface nic = selected.getNic();

                this.txtMAC.Text = string.Join(":", nic.GetPhysicalAddress().GetAddressBytes().Select(b => b.ToString("X2")));

                this.listInterface.Items.Clear();
                foreach (var addr in nic.GetIPProperties().UnicastAddresses)
                {
                    // IP Address
                    item = new ListViewItem(addr.Address.ToString());
                    this.listInterface.Items.Add(item);

                    // Netmask
                    item.SubItems.Add(addr.IPv4Mask.ToString());
                }

            }*/
        }
        
        /****************************************
        * Purpose: Sign In button has been pressed
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-06
        ****************************************/
        private void btnTabSignIn_Click(object sender, EventArgs e)
        {
            this.SetTabIndex(MainForm.TAB_SIGNIN);
        }

        /****************************************
        * Purpose: Device Pool button has been pressed
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-06
        ****************************************/
        private void btnTabDevicePool_Click(object sender, EventArgs e)
        {
            this.SetTabIndex(MainForm.TAB_DEVICEPOOL);
        }

        /****************************************
        * Purpose: Option button has been pressed
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-06
        ****************************************/
        private void btnTabOption_Click(object sender, EventArgs e)
        {
            this.SetTabIndex(MainForm.TAB_OPTION);
        }

        /****************************************
        * Purpose: Usage button has been pressed
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-06
        ****************************************/
        private void btnTabUsage_Click(object sender, EventArgs e)
        {
            this.SetTabIndex(MainForm.TAB_USAGE);
        }

        /****************************************
        * Purpose: Log button has been pressed
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-06
        ****************************************/
        private void btnTabLog_Click(object sender, EventArgs e)
        {
            this.SetTabIndex(MainForm.TAB_LOG);
        }

        /****************************************
        * Purpose: Language button has been pressed
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-06
        ****************************************/
        private void btnTabLanguage_Click(object sender, EventArgs e)
        {
            this.SetTabIndex(MainForm.TAB_LANGUAGE);
        }

        /****************************************
        * Purpose: Help button has been pressed
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-06
        ****************************************/
        private void btnTabHelp_Click(object sender, EventArgs e)
        {
            this.SetTabIndex(MainForm.TAB_HELP);
        }

        /****************************************
        * Purpose: Help button has been pressed
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-06
        ****************************************/
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.m_service.DisconnectVPN();
        }
    }
}
