﻿using Newtonsoft.Json;
using System;
using System.Drawing;
using System.Globalization;
using System.Threading;
using System.Windows.Forms;

namespace VPNClient
{
    public partial class TabSignIn : RoundTab
    {
        delegate void SetTextCallback(string text);

        private VPNCloudService m_service = null;
        private ConfigLoader m_loader = null;
        private ClientConfig m_config = null;

        private static CultureInfo m_ciUS = new CultureInfo("en-US");
        private static CultureInfo m_ciDE = new CultureInfo("de-DE");

        public TabSignIn()
        {
            InitializeComponent();

            /* Initialize the config loader */
            this.m_loader = ConfigLoader.GetInstance();
            this.m_config = this.m_loader.GetConfig();

            this.m_service = VPNCloudService.GetInstance();

            /* Set the default selection method */
            if (this.m_config.type == ActivationType.MANUAL) /* Manual */
            {
                this.rdBtnKey.Checked = false;
                this.rdBtnManual.Checked = true;
            }
            else /* Activation key */
            {
                this.rdBtnKey.Checked = true;
                this.rdBtnManual.Checked = false;

                this.txtKeyFile.Text = this.m_config.keyfile;
            }
            this.rdBtnKey_CheckedChanged(null, null);

            /* Set the default VPN Cloud server address */
            this.txtServer.Text = this.m_config.server;

            /* Load group code and client id */
            if (this.m_config.save_uid)
            {
                this.txtGroupCode.Text = this.m_config.group;
                this.txtClientId.Text = this.m_config.uid;
            }                
            this.ckbRememberLogin.Checked = this.m_config.save_uid;

            /* Load password */
            if (this.m_config.save_pwd)
                this.txtPassword.Text = this.m_config.pwd;
            this.ckbRememberPwd.Checked = this.m_config.save_pwd;

            /* Sign out disabled */
            this.btnSignOut.Enabled = false;
        }

        /****************************************
        * Purpose: Append the message to txtResult
        * Input: message - The message string
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-16
        ****************************************/
        private void AppendMessage(String message)
        {
            /* The same as openvpn log format */
            string text = DateTime.Now.ToString("ddd MMM dd HH:mm:ss yyyy", m_ciUS) + " " + message + "\r\n";

            if (this.txtResult.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(AppendConnectLogText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.txtResult.AppendText(text);
            }
        }

        /****************************************
        * Purpose: Set the server status (for safety Thread used)
        * Input: text - The content of server status
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        private void SetServiceStatusText(string text)
        {
            if (this.txtResult.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetServiceStatusText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.AppendMessage(text);
                this.btnTest.Enabled = true; // Enable the test button
            }
        }

        /****************************************
        * Purpose: Append the connect log text (for safety Thread used)
        * Input: text - The content of vpn log 
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-09-05
        ****************************************/
        private void AppendConnectLogText(string text)
        {
            if (this.txtResult.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(AppendConnectLogText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.txtResult.AppendText(text);
            }
        }

        /****************************************
        * Purpose: Chanage the login method
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-19
        ****************************************/
        private void rdBtnKey_CheckedChanged(object sender, EventArgs e)
        {
            if (this.rdBtnKey.Checked) /* Import from the activation key file */
            {
                this.pnlKeyFile.Enabled = true;
                this.pnlAccount.Enabled = false;
            }
            else /* Signin by user password */
            {
                this.pnlKeyFile.Enabled = false;
                this.pnlAccount.Enabled = true;
            }
        }

        /****************************************
        * Purpose: Check the server status
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-09-05
        ****************************************/
        private void GetVPNOutput()
        {
            while (this.m_service.IsOpenvpnRun())
            {
                string content = this.m_service.GetVPNLog();
                if (content == null)
                {
                    break;
                }
                else
                {
                    this.AppendConnectLogText(content + "\n");

                    if(content.IndexOf("Initialization Sequence Completed") > 0)
                    {
                        this.AppendMessage("\nSign in successfully");

                        MainForm.GetInstance().SetSignedIn(true);
                        this.m_service.SetSignedIn(true);

                        /* Reload the device pool */
                        TabDevicePool.GetInstance().SetupDevicePool();
                    }
                }
                Thread.Sleep(100);
            }
        }
        
        /****************************************
        * Purpose: Check the server status
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        private void CheckServerStatus()
        {
            // Test the VPN Cloud server connectivity
            VPNCloudService service = VPNCloudService.GetInstance();
            service.SetServer(this.txtServer.Text);
            string result = service.CheckService();
            this.SetServiceStatusText(result);
        }

        /****************************************
        * Purpose: RememberLogin checkbox value change
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-19
        ****************************************/
        private void ckbRememberLogin_CheckedChanged(object sender, EventArgs e)
        {
            if (this.ckbRememberLogin.Checked)
            {
                this.m_config.save_uid = true;
                this.m_config.uid = this.txtClientId.Text;
                this.m_config.group = this.txtGroupCode.Text;
            }
            else
            {
                this.m_config.save_uid = false;
                this.m_config.uid = "";
                this.m_config.group = "";
            }

            /* Save activation type and key file */
            if (this.rdBtnKey.Checked)
            {
                this.m_config.type = ActivationType.KEY;
                this.m_config.keyfile = this.txtKeyFile.Text;
            }
            else
            {
                this.m_config.type = ActivationType.MANUAL;
            }

            this.m_loader.SaveConfig(this.m_config);
        }

        /****************************************
        * Purpose: RememberPwd checkbox value change
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-19
        ****************************************/
        private void ckbRememberPwd_CheckedChanged(object sender, EventArgs e)
        {
            if (this.ckbRememberPwd.Checked)
            {
                this.m_config.save_pwd = true;
                this.m_config.pwd = this.txtPassword.Text;
            }
            else
            {
                this.m_config.save_pwd = false;
                this.m_config.pwd = "";
            }

            /* Save activation type and key file */
            if (this.rdBtnKey.Checked)
            {
                this.m_config.type = ActivationType.KEY;
                this.m_config.keyfile = this.txtKeyFile.Text;
            }
            else
            {
                this.m_config.type = ActivationType.MANUAL;
            }

            this.m_loader.SaveConfig(this.m_config);
        }

        /****************************************
        * Purpose: Test if the VPN Cloud server is online
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        private void btnTest_Click(object sender, EventArgs e)
        {
            // Open a HTTPS connection
            if (this.txtServer.TextLength != 0)
            {
                this.btnTest.Enabled = false;
                this.AppendMessage("Test connection to " + this.txtServer.Text + " ...");

                Thread workerThread = new Thread(this.CheckServerStatus);
                workerThread.Start();

                /* Save the server string */
                this.m_config.server = this.txtServer.Text;

                /* Save activation type and key file */
                if (this.rdBtnKey.Checked)
                {
                    this.m_config.type = ActivationType.KEY;
                    this.m_config.keyfile = this.txtKeyFile.Text;
                }
                else
                {
                    this.m_config.type = ActivationType.MANUAL;
                }

                this.m_loader.SaveConfig(this.m_config);
            }
            else
            {
                MessageBox.Show("Please fill up the Remote Connect Server Address",
                    "Miss server address", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        public bool EnableNIC(string interfaceName)
        {
            System.Diagnostics.ProcessStartInfo psi =
                   new System.Diagnostics.ProcessStartInfo("netsh", "interface set interface \"" + interfaceName + "\" enable");
            System.Diagnostics.Process p = new System.Diagnostics.Process();
            p.StartInfo = psi;
            p.StartInfo.CreateNoWindow = true;
            p.StartInfo.UseShellExecute = false;
            p.Start();
            p.WaitForExit();

            return (p.ExitCode == 0);
        }

        public bool DisableNIC(string interfaceName)
        {
            System.Diagnostics.ProcessStartInfo psi =
                new System.Diagnostics.ProcessStartInfo("netsh", "interface set interface \"" + interfaceName + "\" disable");
            System.Diagnostics.Process p = new System.Diagnostics.Process();
            p.StartInfo = psi;
            p.StartInfo.CreateNoWindow = true;
            p.StartInfo.UseShellExecute = false;
            p.Start();
            p.WaitForExit();

            return (p.ExitCode  == 0);
        }

        /****************************************
        * Purpose: Do sign in
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        private void btnSignIn_Click(object sender, EventArgs e)
        {
            String nicName = "Moxa Remote Connect";

            if (this.rdBtnKey.Checked)
            {
                /* Sign in by key */
            }
            else
            {
                /* Sign in by client id and password */
                if (this.txtServer.Text.Length == 0)
                {
                    MessageBox.Show("Please fill out the Remote connect Server Address",
                        "Miss server address", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                else if (this.txtGroupCode.Text.Length == 0)
                {
                    MessageBox.Show("Please fill out the Groud Code",
                        "Miss Group Code", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                else if (this.txtClientId.Text.Length == 0)
                {
                    MessageBox.Show("Please fill out the Client ID or E-Mail",
                        "Miss Client ID or E-Mail", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                else if (this.txtPassword.Text.Length == 0)
                {
                    MessageBox.Show("Please fill out the Password",
                        "Miss Password", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
            }

            ckbRememberLogin_CheckedChanged(null, null);
            ckbRememberPwd_CheckedChanged(null, null);

            this.m_service.SetServer(this.txtServer.Text);
            ActivationResponse result = null;

            if (rdBtnKey.Checked == true)
            {
                result = this.m_service.ActivateByKey(this.txtPassword.Text);
            }   
            else
            {
                result = this.m_service.SignIn(this.txtClientId.Text, this.txtPassword.Text, this.txtGroupCode.Text);
            }   

            if (result.message.Equals("success"))
            {
                /* Disable the client input and sign in */
                this.pnlClient.Enabled = false;
                this.btnSignIn.Enabled = false;
                this.btnSignOut.Enabled = true;

                MainForm.GetInstance().SetVirtualIP(this.m_service.GetClientVirtualIP());

                /* Save the server string */
                this.m_config.server = this.txtServer.Text;

                /* Save activation type and key file */
                if (this.rdBtnKey.Checked)
                {
                    this.m_config.type = ActivationType.KEY;
                    this.m_config.keyfile = this.txtKeyFile.Text;
                }
                else
                {
                    this.m_config.type = ActivationType.MANUAL;
                }

                this.m_loader.SaveConfig(this.m_config);

                /* Connect to the openvpn server */
                this.AppendMessage("Initializing network interface...");
                if(this.DisableNIC(nicName) == false)
                {
                    this.AppendMessage("Failed");
                    btnSignOut_Click(null, null);
                    return;
                }
                if (this.EnableNIC(nicName) == false)
                {
                    this.AppendMessage("Failed");
                    btnSignOut_Click(null, null);
                    return;
                }

                this.m_service.ConnectVPN();
                Thread workerThread = new Thread(this.GetVPNOutput);
                workerThread.Start();
            }
            else
            {
                MessageBox.Show(result.message);
            }
        }

        /****************************************
        * Purpose: Do sign out
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-15
        ****************************************/
        private void btnSignOut_Click(object sender, EventArgs e)
        {
            /* Disconnect the openvpn */
            this.m_service.DisconnectVPN();

            /* Enable the client input and sign in */
            this.pnlClient.Enabled = true;
            this.btnSignIn.Enabled = true;
            this.btnSignOut.Enabled = false;

            if(sender != null)
            {
                /* Trigger nu button */
                this.AppendMessage("Sign out");
            }
            
            this.m_service.SignOut();
            MainForm.GetInstance().SetSignedIn(false);
            MainForm.GetInstance().SetVirtualIP("");
        }
        
        /****************************************
        * Purpose: Set the button enable/disable background color
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-16
        ****************************************/
        private void btn_EnabledChanged(object sender, EventArgs e)
        {
            Button btn = (Button)sender;
            btn.ForeColor = Color.White;
            btn.BackColor = !btn.Enabled ? Color.LightGray : Color.CadetBlue;
        }

        /****************************************
        * Purpose: Redraw the button text to let the disabled button using fore-color text
        * Input: Default UI callback function parameters
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-16
        ****************************************/
        private void btn_Paint(object sender, PaintEventArgs e)
        {
            Button btn = (Button)sender;
            TextFormatFlags flags = TextFormatFlags.HorizontalCenter | TextFormatFlags.VerticalCenter | TextFormatFlags.WordBreak;   // center the text
            TextRenderer.DrawText(e.Graphics, btn.Text, btn.Font, e.ClipRectangle, btn.ForeColor, flags);
        }

        private void btnKeyBrowse_Click(object sender, EventArgs e)
        {
            if (this.fileDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    /* Display the correct activation key filename */
                    txtKeyFile.Text = this.fileDlg.FileName;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Please select the correct activation key", "Invalid Activation Key",
                        MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
        }

        private void btnKeyFile_Click(object sender, EventArgs e)
        {
            try
            {
                System.IO.StreamReader sr = new System.IO.StreamReader(txtKeyFile.Text);
                string text = sr.ReadToEnd();
                sr.Close();

                /* Decode the Base64 key */
                string content = System.Text.Encoding.Default.GetString(System.Convert.FromBase64String(text));

                /* Parse the JSON format key */
                ActivationKey key = JsonConvert.DeserializeObject<ActivationKey>(content);

                this.txtServer.Text = key.server;
                this.txtGroupCode.Text = key.group;
                this.txtClientId.Text = key.name;
                this.txtPassword.Text = text;
                //MessageBox.Show(key.crt);
                //MessageBox.Show(key.key);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Please select the correct activation key", "Invalid Activation Key",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
    }
}
