﻿/*###############################################################################
# Copyright (C) MOXA Inc. All rights reserved.
#
# This software is distributed under the terms of the
# MOXA License.  See the file COPYING-MOXA for details.
#
# Purpose : MOXA PnP VPN Cloud Service for VPN Client operation
# Author : Jemin Su
# Date : 2016-08-05
# Version : 1.0
# Notes :
##############################################################################*/

using Newtonsoft.Json;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace VPNClient
{
    class VPNCloudService
    {
        /* Singleton instance */
        private static readonly VPNCloudService m_instance = new VPNCloudService();

        private Boolean m_bSignedIn = false;

        private string m_server = "";
        private string m_key = "";
        private string m_vip = "";
        private string m_network = "";
        private string m_ovpn = "";
        private Process m_openvpn = null;
        private MD5 m_md5;

        private string m_tmpFileName = "";
        private List<DeviceTreeNode> m_connectedNode = null;

        private List<DeviceTreeNode> m_devicePoolList = null;

        /****************************************
        * Purpose: Constructor for VPNCloudService
        * Input: server - The VPN Cloud Server address
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        private VPNCloudService()
        {
            m_connectedNode = new List<DeviceTreeNode>();

            m_md5 = MD5.Create();
        }

        /****************************************
        * Purpose: Singleton for VPNCloudService
        * Input: N/A
        * Output: N/A
        * Return: VPNCloudService instance
        * Author: Jemin Su
        * Date: 2017-02-15
        ****************************************/
        public static VPNCloudService GetInstance()
        {
            return m_instance;
        }

        /****************************************
        * Purpose: Get the client virtual IP
        * Input: N/A
        * Output: N/A
        * Return: Client virtual ip
        * Author: Jemin Su
        * Date: 2017-02-17
        ****************************************/
        public string GetClientVirtualIP()
        {
            return m_vip;
        }

        /****************************************
        * Purpose: Set the server for VPNCloudService
        * Input: server - The VPN Cloud Server address
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-15
        ****************************************/
        public void SetServer(string server)
        {
            this.m_server = server;
        }

        public void SetDevicePoolList(List<DeviceTreeNode> list)
        {
            /* Direct return if it is not be signed in */
            if(this.m_bSignedIn == false)
            {
                return;
            }

              /* Clean the older ip route */
            for (int i = 0; i < m_connectedNode.Count; i++)
            {
                this.DisconnectDevice((DeviceTreeNode)m_connectedNode[i]);
            }

            this.m_devicePoolList = list;

            /* Connect the device from connected device list */
            for (int i = 0; i < m_devicePoolList.Count; i++)
            {
                DeviceTreeNode gateway = (DeviceTreeNode)m_devicePoolList[i];
                if (gateway.connected)
                {
                    this.ConnectDevice(gateway);
                }
                if (gateway.devices != null)
                {
                    for (int j = 0; j < gateway.devices.Count; j++)
                    {
                        DeviceTreeNode device = (DeviceTreeNode)gateway.devices[j];
                        if (device.connected)
                        {
                            this.ConnectDevice(device);
                        }
                    }
                }
            }
        }

        public List<DeviceTreeNode> GetDevicePoolList()
        {
            return this.m_devicePoolList;
        }

        /****************************************
        * Purpose: Load the activation file
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-25
        ****************************************/
        public ActivationKey LoadActivationKey()
        {
            ActivationKey key = null;
            
            return key;
        }

        /****************************************
        * Purpose: For HTTPS certificate verify
        * Input: Default callback function parameters
        * Output: N/A
        * Return: Always true
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        private bool CheckValidationResult(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors errors)
        {
            return true; // Always accept
        }

        /****************************************
        * Purpose: Check if the server is available
        * Input: N/A
        * Output: N/A
        * Return: The result message of connecting to server
        * Author: Jemin Su
        * Date: 2016-08-05
        ****************************************/
        public string CheckService()
        {
            string sUrl = "https://" + this.m_server + "/api/v1/client/login";
            string sResult = "";
            try
            {
                HttpWebResponse resp = null;
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(sUrl);
                
                ServicePointManager.ServerCertificateValidationCallback =
                    new RemoteCertificateValidationCallback(CheckValidationResult);

                // .NET use the proxy in default setting
                // We should cancel the proxy to make the first resuest quickly
                req.Proxy = null;

                req.Timeout = 30000; // Set the timeout to 30 seconds
                resp = (HttpWebResponse)req.GetResponse();
                if (resp.StatusCode == HttpStatusCode.OK)
                {
                    sResult = this.m_server + " is online";
                }
                else
                {
                    sResult = this.m_server + " is currently under maintenance";
                }

                resp.Close();
            }
            catch (Exception ex)
            {
                sResult = ex.Message;
            }

            return sResult;
        }

        /****************************************
        * Purpose: Activate this client by activation key string
        * Input: key: activation key string
        * Output: N/A
        * Return: Activation Response
        * Author: Jemin Su
        * Date: 2016-08-19
        ****************************************/
        public ActivationResponse ActivateByKey(string key)
        {
            string sUrl = "https://" + this.m_server + "/api/v1/client/login";
            ActivationResponse result = null;

            try
            {
                HttpWebResponse resp = null;
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(sUrl);

                ServicePointManager.ServerCertificateValidationCallback =
                    new RemoteCertificateValidationCallback(CheckValidationResult);

                // .NET use the proxy in default setting
                // We should cancel the proxy to make the first resuest quickly
                req.Proxy = null;

                req.Method = "POST";
                req.KeepAlive = true;
                req.ContentType = "application/x-www-form-urlencoded";
                req.Timeout = 30000; /* Set the timeout to 30 seconds */

                // Post data
                string param = "key=" + key;
                req.ContentLength = param.Length; /* For .NET 3.5 */

                byte[] bs = Encoding.ASCII.GetBytes(param);
                Stream reqStream = req.GetRequestStream();
                {
                    reqStream.Write(bs, 0, bs.Length);
                }

                // Fetch response
                resp = (HttpWebResponse)req.GetResponse();
                if (resp.StatusCode == HttpStatusCode.OK)
                {
                    StreamReader sr = new StreamReader(resp.GetResponseStream());
                    result = JsonConvert.DeserializeObject<ActivationResponse>(sr.ReadToEnd());
                    if (result.error == false)
                    {
                        this.m_key = result.key;
                        this.m_ovpn = result.ovpn;
                        this.m_vip = result.ip;
                        this.m_network = result.network;
                    }
                    else
                    {
                        this.m_key = "";
                        this.m_ovpn = "";
                        this.m_vip = "";
                        this.m_network = result.network;
                    }
                    sr.Close();
                }
                else
                {
                    result = new ActivationResponse();
                    result.error = true;
                    result.message = "Fail to activate client";
                }

                resp.Close();
            }
            catch (Exception ex)
            {
                result = new ActivationResponse();
                result.error = true;
                result.message = ex.Message;
            }

            return result;
        }

        /****************************************
        * Purpose: Sign in by id and password
        * Input: uid:   Login ID
        *        pwd:   Login Password
        *        group: Group code
        * Output: N/A
        * Return: Activation Response
        * Author: Jemin Su
        * Date: 2016-08-21
        ****************************************/
        public ActivationResponse SignIn(string uid, string pwd, string group)
        {
            string sUrl = "https://" + this.m_server + "/api/v1/client/login";
            ActivationResponse result = null;

            try
            {
                HttpWebResponse resp = null;
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(sUrl);

                ServicePointManager.ServerCertificateValidationCallback =
                    new RemoteCertificateValidationCallback(CheckValidationResult);

                /* .NET use the proxy in default setting */
                /* We should cancel the proxy to make the first resuest quickly */
                req.Proxy = null;

                req.Method = "POST";
                req.KeepAlive = true;
                req.ContentType = "application/x-www-form-urlencoded";
                req.Timeout = 30000; // Set the timeout to 30 seconds
                
                /* Post login data */
                string param = "uid=" + uid + "&pwd=" + pwd + "&group=" + group;
                req.ContentLength = param.Length; /* For .NET 3.5 */

                byte[] bs = Encoding.ASCII.GetBytes(param);
                Stream reqStream = req.GetRequestStream();
                {
                    reqStream.Write(bs, 0, bs.Length);
                }

                /* Fetch response */
                resp = (HttpWebResponse)req.GetResponse();
                if (resp.StatusCode == HttpStatusCode.OK)
                {
                    StreamReader sr = new StreamReader(resp.GetResponseStream());
                    result = JsonConvert.DeserializeObject<ActivationResponse>(sr.ReadToEnd());
                    if(result.error == false)
                    {
                        this.m_key = result.key;
                        this.m_ovpn = result.ovpn;
                        this.m_vip = result.ip;
                        this.m_network = result.network;
                    }
                    else
                    {
                        this.m_key = "";
                        this.m_ovpn = "";
                        this.m_vip = result.ip;
                        this.m_network = result.network;
                    }
                    sr.Close();
                }
                else
                {
                    result = new ActivationResponse();
                    result.error = true;
                    result.message = "Fail to login";
                }

                resp.Close();
                resp = null;
            }
            catch (Exception ex)
            {
                result = new ActivationResponse();
                result.error = true;
                result.message = ex.Message;
            }

            return result;
        }

        /****************************************
        * Purpose: Sign out
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-15
        ****************************************/
        public void SignOut()
        {
            this.m_key = "";
            this.m_bSignedIn = false;

            this.m_devicePoolList = null;
        }

        /****************************************
        * Purpose: Get all group devices in the same group
        * Input: N/A
        * Output: N/A
        * Return: DeviceGroup
        * Author: Jemin Su
        * Date: 2016-08-22
        ****************************************/
        public DeviceGroup GetDeviceGroup()
        {
            string sUrl = "https://" + this.m_server + "/api/v1/activation/" + this.m_key + "/group";
            //List<RemoteDevice> devices = null;
            DeviceGroup result = null;

            if (this.m_key.Equals(""))
            {
                result = new DeviceGroup();
                result.error = true;
                result.message = "Please activate client first";
            }
            else
            {
                try
                {
                    HttpWebResponse resp = null;
                    HttpWebRequest req = (HttpWebRequest)WebRequest.Create(sUrl);

                    ServicePointManager.ServerCertificateValidationCallback =
                        new RemoteCertificateValidationCallback(CheckValidationResult);

                    req.Proxy = null;
                    req.Timeout = 30000; // Set the timeout to 30 seconds
                    resp = (HttpWebResponse)req.GetResponse();
                    if (resp.StatusCode == HttpStatusCode.OK)
                    {
                        StreamReader sr = new StreamReader(resp.GetResponseStream());
                        result = JsonConvert.DeserializeObject<DeviceGroup>(sr.ReadToEnd());
                        
                        sr.Close();
                    }
                    else
                    {
                        result = new DeviceGroup();
                        result.error = true;
                        result.message = "Error to fetch device group";
                    }

                    resp.Close();
                }
                catch (Exception ex)
                {
                    result = new DeviceGroup();
                    result.error = true;
                    result.message = ex.Message;
                }
            }

            return result;
        }

        /****************************************
        * Purpose: Build the device virtual ip string
        * Input: N/A
        * Output: N/A
        * Return: DeviceGroup
        * Author: Jemin Su
        * Date: 2016-08-22
        ****************************************/
        public string GetDeviceVirtualIP(int groupNetwork, int gatewayNetwork, int deviceNetwork)
        {
            string ip = null;

            string prefix = "10." + groupNetwork + ".";
            if (gatewayNetwork > 0)
            {
                int ip3 = ((gatewayNetwork - 1) / 8) + 1;
                int ip4 = ((((gatewayNetwork - 1) % 8) << 5) + 1) + deviceNetwork;
                ip = prefix + ip3 + "." + ip4;
            }

            return ip;
        }

        /****************************************
        * Purpose: Build the gateway virtual ip string
        * Input: N/A
        * Output: N/A
        * Return: DeviceGroup
        * Author: Jemin Su
        * Date: 2016-08-22
        ****************************************/
        public string GetGatewayVirtualIP(int groupNetwork, int gatewayNetwork)
        {
            string ip = null;

            string prefix = "10." + groupNetwork + ".";
            if (gatewayNetwork > 0)
            {
                int ip3 = ((gatewayNetwork - 1) / 8) + 1;
                int ip4 = (((gatewayNetwork - 1) % 8) << 5) + 1;
                ip = prefix + ip3 + "." + ip4;
            }

            return ip;
        }

        /****************************************
        * Purpose: Build the client virtual ip string
        * Input: N/A
        * Output: N/A
        * Return: DeviceGroup
        * Author: Jemin Su
        * Date: 2016-08-22
        ****************************************/
        public string GetClientVirtualIP(int groupNetwork, int clientNetwork)
        {
            string ip = null;

            if ((clientNetwork >= 0) && (clientNetwork < 250))
            {
                ip = "10." + groupNetwork + ".0." + (clientNetwork + 4);
            }

            return ip;
        }

        /****************************************
        * Purpose: Set the signed in flag
        * Input: signed - signed in status
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2018-04-17
        ****************************************/
        public void SetSignedIn(bool signed)
        {
            this.m_bSignedIn = signed;
        }

        /****************************************
        * Purpose: Check if the client has been signed in
        * Input: N/A
        * Output: N/A
        * Return: Is signed in
        * Author: Jemin Su
        * Date: 2018-04-16
        ****************************************/
        public bool IsSignedIn()
        {
            return this.m_bSignedIn;
        }

        /****************************************
        * Purpose: Connect the VPN
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-09-04
        ****************************************/
        public void ConnectVPN()
        {
            if(this.m_ovpn.Equals("") == false)
            {
                m_tmpFileName = Guid.NewGuid().ToString();
                string ovpnFilename = System.IO.Path.GetTempPath() + m_tmpFileName + ".ovpn";
                string batFilename = m_tmpFileName + ".bat";
                try
                {
                    System.IO.StreamWriter sw = new System.IO.StreamWriter(System.IO.Path.GetTempPath() + batFilename);
                    sw.Write("route delete " + this.m_network + " mask 255.255.0.0");
                    sw.Close();

                    /* Create the ovpn file */
                    sw = new System.IO.StreamWriter(ovpnFilename);
                    sw.Write(this.m_ovpn);
                    sw.Write("cd \"" + System.IO.Path.GetTempPath().Replace("\\", "\\\\") + "\"\n");
                    sw.Write("up \"" + batFilename + "\"");
                    sw.Flush();
                    sw.Close();

                    /* Start the vpn process */
                    this.m_openvpn = new Process();
                    m_openvpn.StartInfo.FileName = "openvpn";
                    m_openvpn.StartInfo.Arguments = "\"" + ovpnFilename + "\"";
                    m_openvpn.StartInfo.UseShellExecute = false;
                    m_openvpn.StartInfo.RedirectStandardOutput = true;
                    m_openvpn.StartInfo.CreateNoWindow = true;

                    m_openvpn.Start();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
            else
            {
                /* No ovpn file exist */
            }
        }

        /****************************************
        * Purpose: Disconnect the VPN
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-09-04
        ****************************************/
        public void DisconnectVPN()
        {
            if(m_openvpn != null)
            {
                try
                {
                    m_openvpn.WaitForExit(100); // Wait for 100ms before terminal
                    //m_openvpn.CloseMainWindow();
                    m_openvpn.Kill();

                    for(int i = 0; i < m_connectedNode.Count; i++)
                    {
                        this.DisconnectDevice((DeviceTreeNode)m_connectedNode[i]);
                    }
                }
                catch(Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
                finally
                {
                    m_openvpn = null;

                    try
                    {
                        /* Remove the temp */
                        File.Delete(m_tmpFileName + ".ovpn");
                        File.Delete(m_tmpFileName + ".bat");
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex.Message);
                    }
                }
            }
        }

        /****************************************
        * Purpose: Connect devices via gateway VPN IP
        * Input: device - device node
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-03-29
        ****************************************/
        public void ConnectDevice(DeviceTreeNode device)
        {
            if (m_openvpn != null)
            {
                try
                {
                    /* Start the vpn process */
                    Process route = new Process();
                    route.StartInfo.FileName = "route";
                    if(device.gateway != null)
                    {
                        if((device.gateway.auto_mapping == false) && (device.gateway.Devices.Count == 0))
                        {
                            /* Site-to-Site gateway */
                            var ip = IPAddress.Parse(device.gateway.local_ip);
                            var mask = IPAddress.Parse(device.gateway.local_mask);

                            byte[] ipData = ip.GetAddressBytes();
                            byte[] maskData = mask.GetAddressBytes();
                            for(int i = 0; i < ipData.Length; i++)
                            {
                                ipData[i] &= maskData[i];
                            }
                            route.StartInfo.Arguments = "add " + new IPAddress(ipData) + " mask " + device.gateway.local_mask + " " + this.m_vip;
                        }
                        else
                        {
                            /* Gateway always use virtual IP to connect */
                            route.StartInfo.Arguments = "add " + device.vip + " mask 255.255.255.255 " + this.m_vip;
                        }
                    }
                    else
                    {
                        if (device.vip.Equals("-"))
                                route.StartInfo.Arguments = "add " + device.ip + " mask 255.255.255.255 " + this.m_vip;
                        else
                            route.StartInfo.Arguments = "add " + device.vip + " mask 255.255.255.255 " + this.m_vip;
                    }

                    route.StartInfo.UseShellExecute = false;
                    route.StartInfo.RedirectStandardOutput = true;
                    route.StartInfo.CreateNoWindow = true;

                    route.Start();
                    route.WaitForExit();
                    route.Dispose();

                    /* Add the gateway/device into the connected list */
                    m_connectedNode.Add(device);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
        }

        /****************************************
        * Purpose: Disconnect devices via gateway VPN IP
        * Input: device - device node
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-03-29
        ****************************************/
        public void DisconnectDevice(DeviceTreeNode device)
        {
            if (m_openvpn != null)
            {
                try
                {
                    /* Start the vpn process */
                    Process route = new Process();
                    route.StartInfo.FileName = "route";
                    if(device.gateway != null)
                    {
                        if ((device.gateway.auto_mapping == false) && (device.gateway.Devices.Count == 0))
                        {
                            /* Site-to-Site gateway */
                            var ip = IPAddress.Parse(device.gateway.local_ip);
                            var mask = IPAddress.Parse(device.gateway.local_mask);

                            byte[] ipData = ip.GetAddressBytes();
                            byte[] maskData = mask.GetAddressBytes();
                            for (int i = 0; i < ipData.Length; i++)
                            {
                                ipData[i] &= maskData[i];
                            }

                            route.StartInfo.Arguments = "delete " + new IPAddress(ipData) + " mask " + device.gateway.local_mask;
                        }
                        else
                        {
                            route.StartInfo.Arguments = "delete " + device.vip + " mask 255.255.255.255";
                        }
                    }
                    else
                    {
                        if (device.vip.Equals("-"))
                            route.StartInfo.Arguments = "delete " + device.ip + " mask 255.255.255.255";
                        else
                            route.StartInfo.Arguments = "delete " + device.vip + " mask 255.255.255.255";
                    }

                    route.StartInfo.UseShellExecute = false;
                    route.StartInfo.RedirectStandardOutput = true;
                    route.StartInfo.CreateNoWindow = true;

                    route.Start();
                    route.WaitForExit();
                    route.Dispose();

                    /* Remove the gateway/device from connected list */
                    m_connectedNode.Remove(device);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
        }

        /****************************************
        * Purpose: Get the VPN output string
        * Input: N/A
        * Output: N/A
        * Return: VPN log string
        * Author: Jemin Su
        * Date: 2016-09-05
        ****************************************/
        public string GetVPNLog()
        {
            if (m_openvpn != null)
            {
                return m_openvpn.StandardOutput.ReadLine();
            }

            return null;
        }

        /****************************************
        * Purpose: Get the openvpn running status
        * Input: N/A
        * Output: N/A
        * Return: VPN log string
        * Author: Jemin Su
        * Date: 2016-09-05
        ****************************************/
        public bool IsOpenvpnRun()
        {
            return (m_openvpn != null);
        }

        static string GetMd5Hash(MD5 md5Hash, string input)
        {

            // Convert the input string to a byte array and compute the hash.
            byte[] data = md5Hash.ComputeHash(Encoding.UTF8.GetBytes(input));

            // Create a new Stringbuilder to collect the bytes
            // and create a string.
            StringBuilder sBuilder = new StringBuilder();

            // Loop through each byte of the hashed data 
            // and format each one as a hexadecimal string.
            for (int i = 0; i < data.Length; i++)
            {
                sBuilder.Append(data[i].ToString("x2"));
            }

            // Return the hexadecimal string.
            return sBuilder.ToString();
        }

        /****************************************
        * Purpose: Get the snapshot filename
        * Input: N/A
        * Output: N/A
        * Return: Snapshot filename string
        * Author: Jemin Su
        * Date: 2018-04-10
        ****************************************/
        public string GetSnapshotFilename()
        {
            if(this.m_key.Equals("") == true)
            {
                return null;
            }
            else
            {
                return (GetMd5Hash(this.m_md5, this.m_key) + ".lst");
            }
        }
    }
}
